#!/usr/bin/perl -w

# Copyright 2002, Reto Stamm
#
# This file is part of Hopify.
#
# Hopify is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Hopify is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hopify; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Average number of characters per Chunk
my $CHUNK = 6;

# Maximum number of lines per Chunk (0 means no vertical Chunking)
my $VCHUNK = 0;

use strict;
print "\\documentclass[twocolumn]{report}\n";
print "\\title{Christian Science}\n";
print "\\author{Mark Twain}\n";
print "\\begin{document}\n";
print "\\maketitle\n";

print "\\newcommand{\\hbl}[1]{\\makebox[100pt][r]{\\hfill #1}\\makebox[20pt][r]{\\hfill}}";
print "\\newcommand{\\hbr}[1]{\\makebox[100pt][l]{#1 \\hfill}\\hfill}";

my @paragraphs;
my $paragraph = "";

my $CHARCT= int($CHUNK + $CHUNK/4);
my $CHARCT2= $CHUNK;
my $CHARCT3= int($CHUNK -$CHUNK/4);

my $lastnoempty = 0;
foreach my $line (<>) {
  # remove junk
  chomp $line;
  $line =~ s/^ *//g;
  $line =~ s/ *$//g;

  # escape tex meta-chars
  $line =~ s/([_^\\\{\}%&\$\#])/\\$1/g;

  # fix quotes
  $line =~ s/"([\w]+)/``$1/g;
  $line =~ s/([\w]+)"/$1''/g;

  $paragraph = $paragraph ." ". $line;

  if ($line =~ /^ *$/) {
    if ($lastnoempty) {
      if ($paragraph =~ /[a-z]/) {
        print "\\noindent\n";
        my $vchunkcount = 0;
        while (length($paragraph)>1) {
          $paragraph=~s/^ //;
          my ($chunk) = $paragraph=~/(^.{$CHARCT}[^- ]*-*)/;
          my ($chunk2) = $paragraph=~/(^.{$CHARCT2}[^- ]*-*)/;
          my ($chunk3) = $paragraph=~/(^.{$CHARCT3}[^- ]*-*)/;

          if (!defined $chunk) {
            $chunk = $paragraph;
            $chunk2 = $paragraph;
            $chunk3 = $paragraph;
            $paragraph = "";
          }
          if (abs(length($chunk2)-(($CHARCT2+$CHARCT+$CHARCT3)/3)) <
              abs(length($chunk)-(($CHARCT2+$CHARCT+$CHARCT3)/3))) {
            $chunk = $chunk2;
          }
 
          if (abs(length($chunk3)-(($CHARCT2+$CHARCT+$CHARCT3)/3)) <
              abs(length($chunk)-(($CHARCT2+$CHARCT+$CHARCT3)/3))) {
            $chunk = $chunk3;
          }
	  print "\\hbl{$chunk}\n";

	  $chunk =~ s/[\?\\\]\[\(\)\$\#]/./g;

          $paragraph =~ s/ ?$chunk ?// or length($paragraph)==0 or die "Problem matching [$chunk] in [$paragraph]!\n";
          $paragraph=~s/^ //;
          ($chunk) = $paragraph=~/(^.{$CHARCT}[^- ]*-*)/;
          ($chunk2) = $paragraph=~/(^.{$CHARCT2}[^- ]*-*)/;
          ($chunk3) = $paragraph=~/(^.{$CHARCT3}[^- ]*-*)/;
          if (!defined $chunk) {
            $chunk = $paragraph;
            $chunk2 = $paragraph;
            $chunk3 = $paragraph;
            $paragraph = "";
          }

          if (abs(length($chunk2)-(($CHARCT2+$CHARCT+$CHARCT3)/3)) <
              abs(length($chunk)-(($CHARCT2+$CHARCT+$CHARCT3)/3))) {
            $chunk = $chunk2;
          }
 
          if (abs(length($chunk3)-(($CHARCT2+$CHARCT+$CHARCT3)/3)) <
              abs(length($chunk)-(($CHARCT2+$CHARCT+$CHARCT3)/3))) {
            $chunk = $chunk3;
          }

          print "\\hbr{$chunk}";
	  $chunk =~ s/[\?\\\]\[\(\)\$\#]/./g;

          $paragraph =~ s/ ?$chunk ?// or length($paragraph)==0 or die "Problem matching [$chunk] in [$paragraph]!\n";
          $vchunkcount++;
          if ($VCHUNK>0 and $vchunkcount>= $VCHUNK) {
            $vchunkcount = 0;
            print "\\\\\\\\\n";
          } else {
            print "\\\\\n";
          }
        }
        if ($VCHUNK > 0) {
          print "\\\\\n";
        }
        print "\n\n\n\n";
      } else {
        $paragraph =~ s/CHAPTER [^ ]*//;
        print "\n\\chapter{$paragraph}\n\n"
      }
    }
    $paragraph = "";
    $lastnoempty = 0;
  } else {
    $lastnoempty = 1;
  } 

}



print "\\end{document}\n";
